/**********************************************************************
 * $Id: dmt-ux.c 177 2006-09-24 23:00:54Z timo $
 *
 * DMT-UX
 * DSL Modem Tool for Unix
 *
 * written by Timo Boettcher
 * inspired by A. Matthoefer
 * with help of Michael Beckers
 *
 * (C) 2006 Timo Boettcher
 * This is free software. You may redistribute copies of it under the
 * terms of the GNU General Public License (Version 2)
 * <http://www.gnu.org/licenses/gpl.html>.
 * There is NO WARRANTY, to the extent permitted by law.
 *
 *********************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>
#include <netdb.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <math.h>
#include <getopt.h>
#ifdef graphic
#include <png.h>
#include <ft2build.h>
#include FT_FREETYPE_H
#endif //graphic
#ifdef rrd
#include <rrd.h>
#endif //rrd

#include "version.h"
#include "types.h"
#include "helper.h"
#include "modules/speedtouch.h"
#include "dslam-vendors.h"
#include "dsl-standards.h"
#ifdef graphic
#include "colors.h"
#include "graphic.h"
#endif //graphic
#ifdef rrd
#include "rrdstat.h"
#endif //rrd
#include "dmt-ux.h"

#ifdef graphic
#include "images/downstream-arrowhead.xpm"
#include "images/downstream-arrowsection.xpm"
#include "images/modem-local.xpm"
#include "images/modem-remote.xpm"
#include "images/upstream-arrowhead.xpm"
#include "images/upstream-arrowsection.xpm"
#include "images/world.xpm"
#endif //graphic


Modem* modemdata;
IntArray dsl_bitalloc;
IntArray dsl_snr;
IntArray dsl_noisemargin;
IntArray dsl_chancharlog;
unsigned int dsl_errorsecs;
StringList* commandlist;

Config conf;

struct sockaddr_in server;
int sockaddr_size;
struct hostent *hp;

// consts that are configured via commandline switches
#ifdef graphic
unsigned int GRAPH_BPP;
unsigned int GRAPH_WIDTH;
unsigned int BIGGRAPH_WIDTH;
unsigned int MEDIUMGRAPH_WIDTH;
#endif //graphic
unsigned int DROP_TONES;



int main(int argc, char** argv) {
	printf("DMT-UX 0.%i (DSL Modem Tool for Unix)\n", version);
	printf("(C) 2006 by Timo Boettcher\n");
	printf("\n");
	printf("This is free software. You may redistribute copies of it under the terms of the\n");
	printf("GNU General Public License (Version 2) <http://www.gnu.org/licenses/gpl.html>.\n");
	printf("There is NO WARRANTY, to the extent permitted by law.\n");
	printf("\n");
	printf("http://www.spida.net/projects/dmt-ux\n");


	init_dmt(argc, argv);
	pid_t pid;

	if (conf.pidfile) {
		if ((strlen(conf.pidfile)>0)) {
			message_log_open(conf.logfile);
			pid = fork();
			if (pid < 0) {
				fprintf(stderr, "fork failed\n");
				exit(1);
			}
			if (pid == 0) {
				mainloop();
			}
			if (pid > 0) {
				fprintf(stderr, "Running in backgroud, pid is %i\n", (int)pid);
				pidfile_write(conf.pidfile, pid);
				exit(EXIT_SUCCESS);
			}
		} else {
			mainloop();
		}
	} else {
		mainloop();
	}


	message_log_close();
	fprintf (stderr,"completed\n");
	return EXIT_SUCCESS;
}

void mainloop() {
	struct stat statbuf;
	int csock;
#ifdef rrd
	struct timeval tp;
#endif //rrd

	do{
		csock = connect_to_modem();
		init_dmt_data(csock);
		handle_socket(csock);
#ifdef rrd
		gettimeofday(&tp,NULL);
#endif //rrd
		parse_data(commandlist);

		if (conf.information) {
			information_prepare();
		}
#ifdef graphic
		if (stat(conf.font, &statbuf) < 0) {
			char* message = dmalloc(sizeof(char) * 1024);
			sprintf(message, "Couldn't stat Fontfile \"%s\"\n", conf.font);
			message_write(MSGLEVEL_WARNING,message);
			free(message);
			conf.font="";
			conf.legend=0;
		} else {
			if (S_ISREG(statbuf.st_mode) == 0) {
				char* message = dmalloc(sizeof(char) * 1024);
				sprintf(message, "Fontfile \"%s\" is not a file\n", conf.font);
				message_write(MSGLEVEL_WARNING,message);
				free(message);
				conf.font="";
				conf.legend=0;
			}
		}
		if (conf.imagefile && conf.font) {
			if ((strlen(conf.imagefile)>0) && (strlen(conf.font)>0)) {
				graphs_write_all(conf.imagefile);
			}
		}
		if (conf.imagefile_bitalloc) {
			if (strlen(conf.imagefile_bitalloc)>0) {
				graphs_write_bitalloc(conf.imagefile_bitalloc);
			}
		}
		if (conf.imagefile_noise) {
			if (strlen(conf.imagefile_noise)>0) {
				graphs_write_noise(conf.imagefile_noise);
			}
		}
		if (conf.imagefile_chancharlog) {
			if (strlen(conf.imagefile_chancharlog)>0) {
				graphs_write_chancharlog(conf.imagefile_chancharlog);
			}
		}
#endif //graphic
#ifdef rrd
		if (conf.rrdfile_errorsecs) {
			if ((strlen(conf.rrdfile_errorsecs)>0)) {
				if (stat(conf.rrdfile_errorsecs, &statbuf) < 0) {
					char* message = dmalloc(sizeof(char) * 1024);
					sprintf(message, "Couldn't stat Errorsecfile \"%s\". Creating it.\n", conf.rrdfile_errorsecs);
					message_write(MSGLEVEL_WARNING,message);
					free(message);
					rrdstat_create_errorsecs(conf.rrdfile_errorsecs);
				} else {
					if (S_ISREG(statbuf.st_mode) == 0) {
						char* message = dmalloc(sizeof(char) * 1024);
						sprintf(message, "Errorsecfile is not a file \"%s\"\n", conf.rrdfile_errorsecs);
						message_write(MSGLEVEL_WARNING,message);
						free(message);
						conf.rrdfile_errorsecs="";
					}
				}
			}
		}
		if (conf.rrdfile_snr) {
			if ((strlen(conf.rrdfile_snr)>0)) {
				if (stat(conf.rrdfile_snr, &statbuf) < 0) {
					char* message = dmalloc(sizeof(char) * 1024);
					sprintf(message, "Couldn't stat Snrfile \"%s\". Creating it.\n", conf.rrdfile_snr);
					message_write(MSGLEVEL_WARNING,message);
					free(message);
					rrdstat_create_snr(conf.rrdfile_snr);
				} else {
					if (S_ISREG(statbuf.st_mode) == 0) {
						char* message = dmalloc(sizeof(char) * 1024);
						sprintf(message, "Snrfile is not a file \"%s\"\n", conf.rrdfile_snr);
						message_write(MSGLEVEL_WARNING,message);
						free(message);
						conf.rrdfile_snr="";
					}
				}
			}
		}
		if (conf.daemon && conf.rrdfile_errorsecs) {
			if ((strlen(conf.rrdfile_errorsecs)>0)) {
				rrdstat_update_errorsecs(conf.rrdfile_errorsecs,tp,dsl_errorsecs);
			}
		}
		if (conf.daemon && conf.rrdfile_snr) {
			if ((strlen(conf.rrdfile_snr)>0)) {
				rrdstat_update_snr(conf.rrdfile_snr,tp,modemdata->noise_low,modemdata->noise_LQ,modemdata->noise_high);
			}
		}
#ifdef graphic
		if (conf.imagefile_rrd_errorsecs) {
			if (strlen(conf.imagefile_rrd_errorsecs)>0) {
				graphs_write_rrd_errorsecs(conf.imagefile_rrd_errorsecs);
			}
		}
		if (conf.imagefile_rrd_snr) {
			if (strlen(conf.imagefile_rrd_snr)>0) {
				graphs_write_rrd_snr(conf.imagefile_rrd_snr);
			}
		}
#endif //graphic
#endif //rrd
		if (conf.information) {
			information_print();
		}

		cleanup_dmt();
		if (conf.daemon) {
			char* message = dmalloc(sizeof(char) * 1024);
			sprintf(message, "Sleeping %i seconds\n", conf.daemon);
			message_write(MSGLEVEL_WARNING,message);
			free(message);
			sleep(conf.daemon);
		}
	} while (conf.daemon);

}



void init_dmt(int argc, char** argv) {
	// defaults for commandline arguments
	conf.username = "";
	conf.password = "";
	conf.hostname = "";
	conf.port = "23";
	conf.daemon = 0;
	conf.outputfile = "";
	conf.checksync = 0;
	conf.pidfile = "";
	conf.logfile = "";
#ifdef graphic
	GRAPH_BPP=3;
	GRAPH_WIDTH = GRAPH_WIDTH_DMTUX;
	BIGGRAPH_WIDTH = BIGGRAPH_WIDTH_DMTUX;
	MEDIUMGRAPH_WIDTH = MEDIUMGRAPH_WIDTH_DMTUX;
	conf.font = "./fonts/Vera.ttf";
	conf.information = 0;
	conf.legend = 0;
	conf.transparency = 0;
	conf.imagefile = "";
	conf.imagefile_bitalloc = "";
	conf.imagefile_noise = "";
	conf.imagefile_chancharlog = "";
#ifdef rrd
	conf.imagefile_rrd_errorsecs = "";
	conf.imagefile_rrd_snr = "";
#endif //rrd
#endif //graphic
#ifdef rrd
	conf.rrdfile_errorsecs = "";
	conf.rrdfile_snr = "";
#endif //rrd
	DROP_TONES = DROP_TONES_DMTUX;
	
	// parse commandline arguments
	switch (read_args(argc, argv)) {
	case 1:
		// parameter error
		exit(1);
		break;
	case 2:
		// help or version
		exit(0);
		break;
	}

}

void cleanup_dmt() {
	list_kill(commandlist);
	free(commandlist);
	information_kill();
	intarray_kill(&dsl_bitalloc);
	intarray_kill(&dsl_snr);
	intarray_kill(&dsl_noisemargin);
	intarray_kill(&dsl_chancharlog);
}

void init_dmt_data(int sock) {
	commandlist=(StringList*)dmalloc(sizeof(StringList));
	list_init(commandlist);

	information_init();
	intarray_init(&dsl_bitalloc);
	intarray_init(&dsl_snr);
	intarray_init(&dsl_noisemargin);
	intarray_init(&dsl_chancharlog);

	parser_init(conf.username, conf.password, &dsl_bitalloc, &dsl_snr, &dsl_noisemargin, &dsl_chancharlog, commandlist, &dsl_errorsecs, modemdata, &send_socket, sock,  &message_write);
}

int read_args(int argc, char** argv) {
	int c;
#ifdef graphic
	while ((c = getopt_long(argc, argv, "d::hH:iI:lL:o:p:P:tu:v", long_options, NULL)) != -1) {
#endif //graphic
#ifndef graphic
	while ((c = getopt_long(argc, argv, "d::hH:iL:o:p:P:u:v", long_options, NULL)) != -1) {
#endif //not graphic
		switch (c) {
		case 'd':
			if (optarg) {
				if (atoi(optarg)!=0) {
					conf.daemon = atoi(optarg);
				} else {
					conf.daemon = 5;
				}
			} else {
				conf.daemon = 5;
			}
			break;
		case COMMANDLINEOPTION_DMTCOMPATIBLE:
#ifdef graphic
			GRAPH_WIDTH = GRAPH_WIDTH_DMT;
			BIGGRAPH_WIDTH = BIGGRAPH_WIDTH_DMT;
			MEDIUMGRAPH_WIDTH = MEDIUMGRAPH_WIDTH_DMT;
#endif //graphic
			DROP_TONES = DROP_TONES_DMT;
			break;
		case 'h':
			usage();
			return 2;
			break;
		case 'H':
			conf.hostname = optarg;
			break;
		case 'i':
			conf.information = 1;
			break;
		case COMMANDLINEOPTION_PIDFILE:
			conf.pidfile = optarg;
			break;
		case 'L':
			conf.logfile = optarg;
			break;
#ifdef graphic
		case 'l':
			conf.legend = 1;
			break;
		case COMMANDLINEOPTION_FONT:
			conf.font = optarg;
			break;
		case 'I':
			conf.imagefile = optarg;
			break;
		case COMMANDLINEOPTION_IMAGEFILE:
			conf.imagefile = optarg;
			break;
		case COMMANDLINEOPTION_IMAGEFILE_NOISE:
			conf.imagefile_noise = optarg;
			break;
		case COMMANDLINEOPTION_IMAGEFILE_BITALLOC:
			conf.imagefile_bitalloc = optarg;
			break;
		case COMMANDLINEOPTION_IMAGEFILE_CHANCHARLOG:
			conf.imagefile_chancharlog = optarg;
			break;
#ifdef rrd
		case COMMANDLINEOPTION_IMAGEFILE_RRD_ERRORSECS:
			conf.imagefile_rrd_errorsecs = optarg;
			break;
		case COMMANDLINEOPTION_IMAGEFILE_RRD_SNR:
			conf.imagefile_rrd_snr = optarg;
			break;
#endif //rrd
#endif //graphic
		case 'o':
			conf.outputfile = optarg;
			break;
		case 'p':
			conf.password = optarg;
			break;
		case 'P':
			conf.port = optarg;
			break;
#ifdef rrd
		case COMMANDLINEOPTION_RRDFILE_ERRORSECS:
			conf.rrdfile_errorsecs = optarg;
			break;
		case COMMANDLINEOPTION_RRDFILE_SNR:
			conf.rrdfile_snr = optarg;
			break;
#endif //rrd
#ifdef graphic
		case 't':
			conf.transparency = 1;
			GRAPH_BPP=4;
			break;
#endif //graphic
		case 'u':
			conf.username = optarg;
			break;
		case 'v':
			// version is printed everytime anyway
			return 2;
			break;
		default:
			usage();
			return 1;
		}

	}
	return 0;
}

void usage() {
	printf("Usage:\n");
	printf("\n");
	printf("./dmt-ux\n");
	printf("   -d --daemon[$seconds]\n");
	printf("                                daemon mode: keep running and check\n");
	printf("                                every $seconds. Default is 5\n");
	printf("   -H --hostname modem\n");
	printf("                                hostname or IP where the modem is\n");
	printf("                                reachable\n");
	printf("                                Default is speedtouch.lan\n");
	printf("   -P --port 23\n");
	printf("                                telnet-port of the modem \n");
	printf("                                Default is 23\n");
	printf("   -u --username root\n");
	printf("                                username for logging into the modem\n");
	printf("   -p --password secret\n");
	printf("                                password for logging into the modem\n");
#ifdef graphic
	printf("   -I --imagefile file.png\n");
	printf("                                create png imagefile with the supplied \n");
	printf("                                name containing the above three single \n");
	printf("                                graphs\n");
	printf("      --image-bitalloc-file file.png\n");
	printf("                                create png imagefile with the supplied\n");
	printf("                                name containing bit allocation data\n");
	printf("      --image-noise-file file.png\n");
	printf("                                create png imagefile with the supplied\n");
	printf("                                name containing noise data\n");
	printf("      --image-chancharlog-file file.png\n");
	printf("                                create png imagefile with the supplied\n");
	printf("                                name containing charlog data\n");
#ifdef rrd
	printf("      --image-rrd-errorsecs-file file.png\n");
	printf("                                create png imagefile with the supplied\n");
	printf("                                name containing errorsec data\n");
	printf("      --image-rrd-snr-file file.png\n");
	printf("                                create png imagefile with the supplied\n");
	printf("                                name containing snr data\n");
#endif //rrd
	printf("      --pidfile\n");
	printf("                                selects a file to write the pid to\n");
	printf("                                enables daemon-mode\n");
	printf("   -L --logfile\n");
	printf("                                selects a file to write logs to\n");
	printf("   -l --legend\n");
	printf("                                print scales on the single graphs\n");
#endif //graphic
	printf("   -i --information\n");
	printf("                                print out information on the console\n");
	printf("                                and, if used with -I oder --imagefile\n");
	printf("                                paint linestatistics in the iamge\n");
	printf("      --dmt-compatible\n");
	printf("                                prints dmt-compatible graphs\n");
	printf("                                this doesn't display the first 16 tones\n");
	printf("                                and the \n");
#ifdef graphic
	printf("      --font Vera.ttf\n");
	printf("                                font to use for graphics in images\n");
#endif //graphic
#ifdef rrd
	printf("      --rrd-errorsecs-file file.rrd\n");
	printf("                                file with rrd-data for number of seconds\n");
	printf("                                with errors\n");
	printf("                                will be created if not existend\n");
	printf("      --rrd-snr-file file.rrd\n");
	printf("                                file with rrd-data for snr\n");
	printf("                                will be created if not existend\n");
#endif //rrd
	printf("   -h --help\n");
	printf("                                print usage information\n");
	printf("   -v --version\n");
	printf("                                print version\n");
}




int connect_to_modem() {
	int sock;
	sock=socket(AF_INET,SOCK_STREAM,0);
	if (sock<0) {
		message_write(MSGLEVEL_WARNING,"Error opening stream socket\n");
		exit(1);
	}
	
	server.sin_family=AF_INET;
	hp=gethostbyname(conf.hostname);
	if (hp==0) {
		char* message = dmalloc(sizeof(char) * 1024);
		sprintf(message,"%s: unknown host\n",conf.hostname);
		message_write(MSGLEVEL_WARNING,message);
		free(message);
		exit(2);
	}
	
	memcpy((char *)&server.sin_addr,(char*)hp->h_addr,hp->h_length);
	int port = atoi(conf.port);

	if (port <= 0) {
		char* message = dmalloc(sizeof(char) * 1024);
		sprintf(message,"unknown port %s\n", conf.port);
		message_write(MSGLEVEL_WARNING,message);
		free(message);
		exit(3);
	}
	server.sin_port=htons(port);

	char* message = dmalloc(sizeof(char) * 1024);
	sprintf(message, "Connecting to %s (%s) on port %i...\n",conf.hostname,inet_ntoa(server.sin_addr),port);
	message_write(MSGLEVEL_WARNING,message);
	free(message);

	if (connect(sock,(struct sockaddr *)&server, sizeof(server))<0) {
		message_write(MSGLEVEL_WARNING,"Error connecting stream socket\n");
		exit(4);
	}
	return sock;
}


void handle_socket(int csock) {
	struct timeval timeout;
	fd_set rread;
	int socketopen=1;
	int sr,rval;
	int handledatacommand;

	char * buf=dmalloc(3000*sizeof(char));
	char* rcv_data = dmalloc(sizeof(char));

	do {
		FD_ZERO(&rread);
		FD_SET(csock,&rread);

		memset((char *)&timeout,0,sizeof(timeout));
		timeout.tv_sec=5;
		
		sr=select(csock+1, &rread, (fd_set *)0, (fd_set *)0, &timeout);
		if (sr < 0) {
			perror("select");
			continue;
		}
		if (sr > 0) {
			if (FD_ISSET(csock,&rread)) {
				if ((rval=read(csock,buf,2999)) <= 0) {
					message_write(MSGLEVEL_WARNING,"Connection closed\n");
					socketopen=0;			
				} else {
					if (rval > 0) {
						buf[rval]=0;
						char* new_rcv_data = dmalloc((strlen(buf) + strlen(rcv_data) + 1) * sizeof(char));
						strcpy(new_rcv_data, rcv_data);
						strcat(new_rcv_data, buf);
						free(rcv_data);
						rcv_data = dmalloc((strlen(new_rcv_data) + 1) * sizeof(char)) ;
						strcpy(rcv_data, new_rcv_data);
						free(new_rcv_data);

						//printf("S=%s\n",buf);
						handledatacommand = handle_data(rcv_data) ;
						if (handledatacommand) {
							list_add(commandlist, rcv_data, handledatacommand - 1);
							free(rcv_data);
							rcv_data = dmalloc(sizeof(char));
						}
					}
				}
			}
		}

	} while (socketopen);

	free(buf);
	if (rcv_data) {
		free(rcv_data);
	}
}



void send_socket(int csock, char* data) {
	fd_set wwrite;
	struct timeval timeout;
	int sr;

	do {
		FD_ZERO(&wwrite);
		FD_SET(csock,&wwrite);
	
		memset((char *)&timeout,0,sizeof(timeout));
		timeout.tv_sec=5;
	
		sr=select(csock+1, (fd_set *)0, &wwrite, (fd_set *)0, &timeout);
		if (sr < 0) {
			perror("select");
			continue;
		}
	} while (sr<0);

	if (FD_ISSET(csock,&wwrite)) {
		send(csock,data,strlen(data),0);
	}

}

void information_prepare() {
	tr(modemdata->modemvendor);
	tr(modemdata->modemname);
	tr(modemdata->modemnumber);
	tr(modemdata->modemboardname);
	tr(modemdata->modembootloader);
	tr(modemdata->modemfirmware);
	tr(modemdata->modemserial);
	tr(modemdata->modemmacaddr);
	tr(modemdata->modemoperationmode);
	tr(modemdata->modemchannelmode);
	tr(modemdata->modemstate);
	modemdata->downstream_cells=get_firstvalue(modemdata->downstream);
	modemdata->downstream_bytes=get_secondvalue(modemdata->downstream);
	modemdata->upstream_cells=get_firstvalue(modemdata->upstream);
	modemdata->upstream_bytes=get_secondvalue(modemdata->upstream);
	modemdata->modemattenuation_down=get_firstvalue(modemdata->modemattenuation);
	modemdata->modemattenuation_up=get_secondvalue(modemdata->modemattenuation);
	modemdata->modemmargin_down=get_firstvalue(modemdata->modemmargin);
	modemdata->modemmargin_up=get_secondvalue(modemdata->modemmargin);
	modemdata->modemoutputpower_down=get_firstvalue(modemdata->modemoutputpower);
	modemdata->modemoutputpower_up=get_secondvalue(modemdata->modemoutputpower);
	modemdata->revision_local=get_firstvalue(modemdata->revision);
	modemdata->revision_remote=get_secondvalue(modemdata->revision);
	modemdata->vendor_local=get_firstvalue(modemdata->vendor);
	modemdata->vendor_remote=get_secondvalue(modemdata->vendor);
	modemdata->vendorspecific_local=get_firstvalue(modemdata->vendorspecific);
	modemdata->vendorspecific_remote=get_secondvalue(modemdata->vendorspecific);

	modemdata->noise_low=100000;
	modemdata->noise_high=0;
	modemdata->noise_LQ=0;
	unsigned int LQCount;
	LQCount=0;

	unsigned int i;
	for (i=DROP_TONES; i < dsl_snr.last; i++) {
		LQCount++;
		modemdata->noise_LQ+=(int)dsl_snr.data[i];
		if ((int)dsl_snr.data[i]<modemdata->noise_low && (int)dsl_snr.data[i]!=0) {
			modemdata->noise_low=(int)dsl_snr.data[i];
		}
		if ((int)dsl_snr.data[i]>modemdata->noise_high) {
			modemdata->noise_high=(int)dsl_snr.data[i];
		}
	}
	modemdata->noise_LQ = (int)(modemdata->noise_LQ/LQCount);
	modemdata->linelength=linelength();
}

void information_print() {
	if (conf.logfile) {
		if (strlen(conf.logfile)>0) {
			return;
		}
	}
	printf("modemvendor=%s\n",modemdata->modemvendor);
	printf("modemname=%s\n",modemdata->modemname);
	printf("modemnumber=%s\n",modemdata->modemnumber);
	printf("modemboardname=%s\n",modemdata->modemboardname);
	printf("modembootloader=%s\n",modemdata->modembootloader);
	printf("modemfirmware=%s\n",modemdata->modemfirmware);
	printf("modemserial=%s\n",modemdata->modemserial);
	printf("modemmacaddr=%s\n",modemdata->modemmacaddr);
	printf("modemoperationmode=%s (%s)\n",modemdata->modemoperationmode,dslstandard(modemdata->modemoperationmode));
	printf("modemchannelmode=%s\n",modemdata->modemchannelmode);
	printf("modemstate=%s\n",modemdata->modemstate);
	printf("downstream=%s\n",modemdata->downstream_bytes);
	printf("upstream=%s\n",modemdata->upstream_bytes);
	printf("modemattenuation up=%s\n",modemdata->modemattenuation_up);
	printf("modemattenuation down=%s\n",modemdata->modemattenuation_down);
	printf("modemmargin up=%s\n",modemdata->modemmargin_up);
	printf("modemmargin down=%s\n",modemdata->modemmargin_down);
	printf("modemoutputpower up=%s\n",modemdata->modemoutputpower_up);
	printf("modemoutputpower down=%s\n",modemdata->modemoutputpower_down);
	printf("revision=%s\n",modemdata->revision_remote);
	printf("vendor=%s\n",dslamvendor(modemdata->vendor_remote));
	printf("vendorspecific=%s\n",modemdata->vendorspecific_remote);

	printf("estimated line length %s\n",modemdata->linelength);
}

char* linelength() {
	int tone231;
	int linelength_min;
	int linelength_max;

	char* text;
	tone231=(int)dsl_chancharlog.data[231];
	if (tone231>0) {
		//search minimum chancharlog value
		unsigned int i;
		int chancharlogmin;
		chancharlogmin=100000;
		for (i=DROP_TONES; i < dsl_chancharlog.last; i++) {
			if ((int)dsl_chancharlog.data[i]>0) {
				if ((int)dsl_chancharlog.data[i]<chancharlogmin) {
					chancharlogmin=(int)dsl_chancharlog.data[i];
				}
			}
		}
		//(MD_CHANCHARLOG_TABLE[232]*1000)/20.5 *((low_Misc_Value - 50)+100)/100)
		linelength_min= (int)(tone231 * 10/20.5 * (chancharlogmin + 50));
		//(MD_CHANCHARLOG_TABLE[232]*1000)/13.4)
		linelength_max= (int)(tone231 * 1000/13.4);

		char* text;
		text=dmalloc(sizeof(char)*(8 + log(linelength_min) + log(linelength_max)));
		sprintf(text, "%im - %im",(int)linelength_min,(int)linelength_max);
		return text;
	} else {
		text=dmalloc(sizeof(char)*(26));
		sprintf(text, "couldn't estimate line length");
		return text;
	}
}

void information_init() {
	modemdata = (Modem*)dmalloc(sizeof(Modem));
}

void information_kill() {
	free(modemdata->modemvendor);
	free(modemdata->modemname);
	free(modemdata->modemnumber);
	free(modemdata->modemboardname);
	free(modemdata->modembootloader);
	free(modemdata->modemfirmware);
	free(modemdata->modemserial);
	free(modemdata->modemmacaddr);
	free(modemdata->linelength);
	free(modemdata->modemoperationmode);
	free(modemdata->modemchannelmode);
	free(modemdata->modemstate);
	free(modemdata->downstream);
	free(modemdata->downstream_cells);
	free(modemdata->downstream_bytes);
	free(modemdata->upstream);
	free(modemdata->upstream_cells);
	free(modemdata->upstream_bytes);
	free(modemdata->modemattenuation);
	free(modemdata->modemattenuation_up);
	free(modemdata->modemattenuation_down);
	free(modemdata->modemmargin);
	free(modemdata->modemmargin_up);
	free(modemdata->modemmargin_down);
	free(modemdata->modemoutputpower);
	free(modemdata->modemoutputpower_up);
	free(modemdata->modemoutputpower_down);
	free(modemdata->revision);
	free(modemdata->revision_local);
	free(modemdata->revision_remote);
	free(modemdata->vendor);
	free(modemdata->vendor_local);
	free(modemdata->vendor_remote);
	free(modemdata->vendorspecific);
	free(modemdata->vendorspecific_local);
	free(modemdata->vendorspecific_remote);
	free(modemdata);
}

unsigned int get_bitalloc_upstream_downstream(IntArray bitalloc) {
	// Find the division between upstream and downstream.
	// This seems to be a part of at least two tones near
	// tone 60 without any bits allocated.

	unsigned int i;
	if (bitalloc.last>120) {
		for (i=0; i < 60; i++) {
			if ((bitalloc.data[60-i]==0) && (bitalloc.data[60-i+1]==0)) {
				return (60-i);
			}
			if ((bitalloc.data[60+i]==0) && (bitalloc.data[60+i-1]==0)) {
				return (60+i);
			}
		}
	} else {
		if (bitalloc.last>1) {
			for (i=0; i < (int)(bitalloc.last/2); i++) {
				if ((bitalloc.data[(int)(bitalloc.last/2)-i]==0) && (bitalloc.data[(int)(bitalloc.last/2)-i+1]==0)) {
					return ((int)(bitalloc.last/2)-i);
				}
				if ((bitalloc.data[(int)(bitalloc.last/2)+i]==0) && (bitalloc.data[(int)(bitalloc.last/2)+i-1]==0)) {
					return ((int)(bitalloc.last/2)+i);
				}
			}
		} else {
			message_write(MSGLEVEL_WARNING,"Cannot decide on upstream/downstream division, as there are no bits allocated at all.\n");
		}
	}
	message_write(MSGLEVEL_WARNING,"Couldn't decide on upstream/downstream division.\n");
	return 0;
}

/*****************************************
 * GRAPH Functions
 ****************************************/

#ifdef graphic
void graphs_write_all(char* imagefile) {
	unsigned int BIGGRAPH_HEAD_HEIGHT;
	Image graph;

	if (conf.information) {
		BIGGRAPH_HEAD_HEIGHT = BIGGRAPH_HEAD;
	} else {
		BIGGRAPH_HEAD_HEIGHT = 0;
	}


	image_init(&graph, BIGGRAPH_WIDTH, BIGGRAPH_HEIGHT + BIGGRAPH_HEAD_HEIGHT, GRAPH_BPP);
	image_clear_alpha(&graph, (Color)color_bg, (Color)alpha_invisible);
	
	if (conf.information) {
		unsigned int i;
		char* text;

		image_paint_xpm(&graph, world_xpm, 250, 40);
		for (i=0; i<300; i++) {
			image_paint_xpm(&graph, downstream_arrowsection_xpm, 120+i, 32);
			image_paint_xpm(&graph, upstream_arrowsection_xpm, 120+i, 72);
		}
		image_paint_xpm(&graph, downstream_arrowhead_xpm, 108, 27);
		image_paint_xpm(&graph, upstream_arrowhead_xpm, 420, 68);
		image_paint_xpm(&graph, modem_local_xpm, 70, 47);
		image_paint_xpm(&graph, modem_remote_xpm, 440, 43);

		text=dmalloc(sizeof(char)*(65));
		image_paint_text(&graph, (20) *64, (15) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, "Downstream");
		if (strlen(modemdata->downstream_bytes)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "Bitrate %s kbit/s", modemdata->downstream_bytes);
			image_paint_text(&graph, (120) *64, (15) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
		if (strlen(modemdata->modemattenuation_down)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "Attenuation %s dB", modemdata->modemattenuation_down);
			image_paint_text(&graph, (209) *64, (15) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
		if (strlen(modemdata->modemmargin_down)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "SNR %s dB", modemdata->modemmargin_down);
			image_paint_text(&graph, (304) *64, (15) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
		if (strlen(modemdata->modemoutputpower_down)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "tx pwr %s dBm", modemdata->modemoutputpower_down);
			image_paint_text(&graph, (370) *64, (15) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}

		sprintf(text, "%s", modemdata->modemoperationmode);
		image_paint_text(&graph, (120) *64, (43) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		if (strlen(modemdata->linelength)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "est. line length %s",modemdata->linelength);
			image_paint_text(&graph, (120) *64, (60) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
		sprintf(text, "%s", dslstandard(modemdata->modemoperationmode));
		image_paint_text(&graph, (300) *64, (43) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		if (strlen(modemdata->modemchannelmode)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "%s path", modemdata->modemchannelmode);
			image_paint_text(&graph, (300) *64, (60) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}


		image_paint_text(&graph, (20) *64, (85) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, "Upstream");
		if (strlen(modemdata->upstream_bytes)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "Bitrate %s kbit/s", modemdata->upstream_bytes);
			image_paint_text(&graph, (120) *64, (85) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
		if (strlen(modemdata->modemattenuation_up)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "Attenuation %s dB", modemdata->modemattenuation_up);
			image_paint_text(&graph, (209) *64, (85) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
		if (strlen(modemdata->modemmargin_up)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "SNR %s dB", modemdata->modemmargin_up);
			image_paint_text(&graph, (304) *64, (85) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
		if (strlen(modemdata->modemoutputpower_up)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "tx pwr %s dBm", modemdata->modemoutputpower_up);
			image_paint_text(&graph, (370) *64, (85) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}


		sprintf(text, "%s", dslamvendor(modemdata->vendor_remote));
		image_paint_text(&graph, (440) *64, (30) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		if (strlen(modemdata->vendorspecific_remote)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "Vendorspec. %s", modemdata->vendorspecific_remote);
			image_paint_text(&graph, (440) *64, (72) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
		if (strlen(modemdata->revision_remote)>32) {
			message_write(MSGLEVEL_WARNING, "text too long. something went wrong.\n");
		} else {
			sprintf(text, "Rev: %s", modemdata->revision_remote);
			image_paint_text(&graph, (490) *64, (50) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}

		free(text);
	}

	graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH1_TOP + BIGGRAPH_HEAD_HEIGHT, 32, 32, 1, "Tone");
	graphs_create_bitalloc(&graph, BIGGRAPH_GRAPH_LEFT, BIGGRAPH_GRAPH1_TOP + BIGGRAPH_HEAD_HEIGHT, 1);
	graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT -3, BIGGRAPH_GRAPH1_TOP + BIGGRAPH_HEAD_HEIGHT, GRAPH_HEIGHT, 12, 8, BITALLOC_SCALEFACTOR);
	graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT + GRAPH_WIDTH + 16, BIGGRAPH_GRAPH1_TOP + BIGGRAPH_HEAD_HEIGHT, GRAPH_HEIGHT, 12, 8, BITALLOC_SCALEFACTOR);
	graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH2_TOP + BIGGRAPH_HEAD_HEIGHT, 32, 32, TONE_KHZ_SCALEFACTOR, "kHz");
	graphs_create_noise(&graph, BIGGRAPH_GRAPH_LEFT, BIGGRAPH_GRAPH2_TOP + BIGGRAPH_HEAD_HEIGHT, 1);
	graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT -3, BIGGRAPH_GRAPH2_TOP + BIGGRAPH_HEAD_HEIGHT, GRAPH_HEIGHT, 10, 10, NOISE_SCALEFACTOR);
	graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT + GRAPH_WIDTH + 16, BIGGRAPH_GRAPH2_TOP + BIGGRAPH_HEAD_HEIGHT, GRAPH_HEIGHT, 10, 10, NOISE_SCALEFACTOR);
	graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH3_TOP + BIGGRAPH_HEAD_HEIGHT, 32, 32, 1, "Tone");
	graphs_create_chancharlog(&graph, BIGGRAPH_GRAPH_LEFT, BIGGRAPH_GRAPH3_TOP + BIGGRAPH_HEAD_HEIGHT, 1);
	graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT -3, BIGGRAPH_GRAPH3_TOP + BIGGRAPH_HEAD_HEIGHT, GRAPH_HEIGHT, 10, 10, CHANCHARLOG_SCALEFACTOR);
	graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT + GRAPH_WIDTH + 16, BIGGRAPH_GRAPH3_TOP + BIGGRAPH_HEAD_HEIGHT, GRAPH_HEIGHT, 10, 10, CHANCHARLOG_SCALEFACTOR);
	graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH4_TOP + BIGGRAPH_HEAD_HEIGHT, 32, 32, TONE_KHZ_SCALEFACTOR, "kHz");

	image_save_png(&graph, imagefile);
	image_kill(&graph);
}


void graphs_write_bitalloc(char* imagefile) {
	Image graph;
	if (conf.legend) {
		image_init(&graph, MEDIUMGRAPH_WIDTH, MEDIUMGRAPH_HEIGHT, GRAPH_BPP);
		image_clear_alpha(&graph, (Color)color_bg, (Color)alpha_invisible);

		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH1_TOP, 32, 32, 1, "Tone");
		graphs_create_bitalloc(&graph, BIGGRAPH_GRAPH_LEFT, BIGGRAPH_GRAPH1_TOP, 0);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT -3, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 12, 8, BITALLOC_SCALEFACTOR);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT + GRAPH_WIDTH + 16, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 12, 8, BITALLOC_SCALEFACTOR);
		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH2_TOP, 32, 32, TONE_KHZ_SCALEFACTOR, "kHz");
	} else {
		image_init(&graph, GRAPH_WIDTH, GRAPH_HEIGHT, GRAPH_BPP);
		graphs_create_bitalloc(&graph, 0, 0, 0);
	}

	image_save_png(&graph, imagefile);
	image_kill(&graph);
}
void graphs_create_bitalloc(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text) {
	image_paint_rect(graph, x_start, y_start, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_bg);
	image_paint_grid2(graph, x_start, y_start, GRAPH_BITALLOC_GRID_WIDTH, GRAPH_BITALLOC_GRID_HEIGHT, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_grid);

	unsigned int i;
	unsigned int dsl_upstream_downstream;
	dsl_upstream_downstream = get_bitalloc_upstream_downstream(dsl_bitalloc);
	for (i=DROP_TONES; i < dsl_bitalloc.last; i++) {
		if ((int)dsl_bitalloc.data[i]>0) {
			if (i < dsl_upstream_downstream) {
				image_paint_vert_line(graph, x_start + i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ BITALLOC_FACTOR*(int)dsl_bitalloc.data[i],0,GRAPH_HEIGHT), y_start + GRAPH_HEIGHT - 1, (Color)color_graph_bitalloc_upstream);
			} else {
				image_paint_vert_line(graph, x_start + i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ BITALLOC_FACTOR*(int)dsl_bitalloc.data[i],0,GRAPH_HEIGHT), y_start + GRAPH_HEIGHT -1, (Color)color_graph_bitalloc_downstream);
			}
		}
	}

	if (paint_text) {
		image_paint_text(graph, (x_start + 3) * 64, (y_start + 3) * 64, 0, 6*64, 7*64, conf.font, (Color)color_white, "Bit Loading Values   bits per tone");
	}
}


void graphs_write_noise(char* imagefile) {
	Image graph;
	if (conf.legend) {
		image_init(&graph, MEDIUMGRAPH_WIDTH, MEDIUMGRAPH_HEIGHT, GRAPH_BPP);
		image_clear_alpha(&graph, (Color)color_bg, (Color)alpha_invisible);

		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH1_TOP, 32, 32, 1, "Tone");
		graphs_create_noise(&graph, BIGGRAPH_GRAPH_LEFT, BIGGRAPH_GRAPH1_TOP, 0);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT -3, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 10, 10, NOISE_SCALEFACTOR);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT + GRAPH_WIDTH + 16, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 10, 10, NOISE_SCALEFACTOR);
		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH2_TOP, 32, 32, TONE_KHZ_SCALEFACTOR, "kHz");

	} else {
		image_init(&graph, GRAPH_WIDTH, GRAPH_HEIGHT, GRAPH_BPP);
		graphs_create_noise(&graph, 0, 0, 0);

	}
	image_save_png(&graph, imagefile);
	image_kill(&graph);
}
void graphs_create_noise(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text) {
	image_paint_rect(graph, x_start, y_start, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_bg);
	image_paint_grid2(graph, x_start, y_start, GRAPH_NOISE_GRID_WIDTH, GRAPH_NOISE_GRID_HEIGHT, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_grid);

	unsigned int i;
	for (i=DROP_TONES; i < dsl_snr.last; i++) {
		if ((int)dsl_snr.data[i]>0) {
			image_paint_vert_line(graph, x_start + i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ SNR_FACTOR * (int)dsl_snr.data[i],0,GRAPH_HEIGHT), y_start + GRAPH_HEIGHT -1, (Color)color_graph_snr);
		}
	}
	for (i=DROP_TONES; i < dsl_noisemargin.last; i++) {
		if ((int)dsl_noisemargin.data[i]>0) {
			image_paint_vert_line(graph, x_start + i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ NOISEMARGIN_FACTOR * (int)dsl_noisemargin.data[i],0,GRAPH_HEIGHT), y_start + GRAPH_HEIGHT -1, (Color)color_graph_noisemargin);
		}
	}

	if (paint_text) {
		char* text;
		text=dmalloc(sizeof(char)*(1+log(modemdata->noise_high)));

		sprintf(text, "%i", modemdata->noise_LQ);
		image_paint_text(graph, (x_start + 3) *64, (y_start + 32) *64, 0, 6*64, 7*64, conf.font, (Color)color_graph_noisetext_LQ2, "LQ2");
		image_paint_text(graph, (x_start + 25) *64, (y_start + 32) *64, 0, 6*64, 7*64, conf.font, (Color)color_graph_noisetext_LQ2, text);
		sprintf(text, "%i", modemdata->noise_high);
		image_paint_text(graph, (x_start + 3) *64, (y_start + 42) *64, 0, 6*64, 7*64, conf.font, (Color)color_graph_noisetext, "High");
		image_paint_text(graph, (x_start + 25) *64, (y_start + 42) *64, 0, 6*64, 7*64, conf.font, (Color)color_graph_noisetext, text);
		sprintf(text, "%i", modemdata->noise_low);
		image_paint_text(graph, (x_start + 3) *64, (y_start + 52) *64, 0, 6*64, 7*64, conf.font, (Color)color_graph_noisetext, "Low");
		image_paint_text(graph, (x_start + 25) *64, (y_start + 52) *64, 0, 6*64, 7*64, conf.font, (Color)color_graph_noisetext, text);
		free(text);
		image_paint_text(graph, (x_start + 3) *64, (y_start + 3) *64, 0, 6*64, 7*64, conf.font, (Color)color_white, "Signal-Noise-Ratio   dB per tone");
	}

}


void graphs_write_chancharlog(char* imagefile) {
	Image graph;
	if (conf.legend) {
		image_init(&graph, MEDIUMGRAPH_WIDTH, MEDIUMGRAPH_HEIGHT, GRAPH_BPP);
		image_clear_alpha(&graph, (Color)color_bg, (Color)alpha_invisible);

		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH1_TOP, 32, 32, 1, "Tone");
		graphs_create_chancharlog(&graph, BIGGRAPH_GRAPH_LEFT, BIGGRAPH_GRAPH1_TOP, 0);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT -3, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 10, 10, CHANCHARLOG_SCALEFACTOR);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT + GRAPH_WIDTH + 16, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 10, 10, CHANCHARLOG_SCALEFACTOR);
		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH2_TOP, 32, 32, TONE_KHZ_SCALEFACTOR, "kHz");
	} else {
		image_init(&graph, GRAPH_WIDTH, GRAPH_HEIGHT, GRAPH_BPP);
		graphs_create_chancharlog(&graph, 0, 0, 0);
	}

	image_save_png(&graph, imagefile);
	image_kill(&graph);
}
void graphs_create_chancharlog(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text) {
	image_paint_rect(graph, x_start, y_start, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_bg);
	image_paint_grid2(graph, x_start, y_start, GRAPH_CHANCHARLOG_GRID_WIDTH, GRAPH_CHANCHARLOG_GRID_HEIGHT, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_grid);

	unsigned int i;
	for (i=DROP_TONES; i < dsl_chancharlog.last; i++) {
		if ((int)dsl_chancharlog.data[i]>0) {
			image_paint_vert_line(graph, x_start + i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ CHANCHARLOG_FACTOR * (int)dsl_chancharlog.data[i],0,GRAPH_HEIGHT) , y_start + GRAPH_HEIGHT -1, (Color)color_graph_chancharlog);
		}
	}

	if (paint_text) {
		image_paint_text(graph, (x_start + 3) *64, (y_start + 3) *64, 0, 6*64, 7*64, conf.font, (Color)color_white, "attenuation   dB per tone");
	}
}

#ifdef rrd
void graphs_write_rrd_errorsecs(char* imagefile) {
	Image graph;
	if (conf.legend) {
		image_init(&graph, MEDIUMGRAPH_WIDTH, MEDIUMGRAPH_HEIGHT, GRAPH_BPP);
		image_clear_alpha(&graph, (Color)color_bg, (Color)alpha_invisible);

		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH1_TOP, -20, 60, 0.5, "Minutes");
		graphs_create_rrd_errorsecs(&graph, BIGGRAPH_GRAPH_LEFT, BIGGRAPH_GRAPH1_TOP, 0);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT -3, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 10, 10, ERRORSECS_SCALEFACTOR);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT + GRAPH_WIDTH + 16, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 10, 10, ERRORSECS_SCALEFACTOR);
		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH2_TOP, -20, 60, 0.5, "Minutes");
	} else {
		image_init(&graph, GRAPH_WIDTH, GRAPH_HEIGHT, GRAPH_BPP);
		graphs_create_rrd_errorsecs(&graph, 0, 0, 0);
	}

	image_save_png(&graph, imagefile);
	image_kill(&graph);
}
void graphs_create_rrd_errorsecs(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text) {
	unsigned int datapoint;
	unsigned int dataset;
	unsigned int graphpoint;;
	unsigned int i;
	time_t starttime;
	time_t  endtime;
	unsigned long steptime;
	unsigned long dscount;
	rrd_value_t* rrddata;
	rrd_value_t* rrddatacopy;
	char **dsnames;
	float value;

	rrdstat_fetch_errorsecs(conf.rrdfile_errorsecs, &rrddata, &starttime, &endtime, &steptime, &dscount, &dsnames);
	rrddatacopy=rrddata;

	unsigned int x_offset=0;
	unsigned int datapointscount=(endtime-starttime)/steptime;
	unsigned int stretchfactor=1;
	if (datapointscount<=GRAPH_WIDTH) {
		stretchfactor=(int)(GRAPH_WIDTH/datapointscount);
		x_offset=(GRAPH_WIDTH-(datapointscount*stretchfactor))/2;
	}

	image_paint_rect(graph, x_start + x_offset, y_start, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_bg);
	image_paint_grid2(graph, x_start + x_offset, y_start, GRAPH_ERRORSECS_GRID_WIDTH, GRAPH_ERRORSECS_GRID_HEIGHT, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_grid);

	for (datapoint=starttime+steptime; datapoint<endtime; datapoint+=steptime) {
		for (dataset=0; dataset<dscount; dataset++) {
			value=*(rrddatacopy++);
			graphpoint = (int)((datapoint -starttime)/steptime);
			for (i=0;i<ERRORSECS_STRECH_FACTOR;i++) {
				image_paint_vert_line(graph, x_start + x_offset + (graphpoint*ERRORSECS_STRECH_FACTOR)+i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ (int)(ERRORSECS_FACTOR * value),0,GRAPH_HEIGHT) , y_start + GRAPH_HEIGHT -1, (Color)color_graph_errorsecs);
			}
		}
	}

	for (dataset = 0; dataset < dscount; dataset++) {
		free(dsnames[dataset]);
	}
	free(dsnames);
	free (rrddata);

	if (paint_text) {
		image_paint_text(graph, (x_start + 3) *64, (y_start + 3) *64, 0, 6*64, 7*64, conf.font, (Color)color_white, "errored seconds");
	}
}

void graphs_write_rrd_snr(char* imagefile) {
	Image graph;
	if (conf.legend) {
		image_init(&graph, MEDIUMGRAPH_WIDTH, MEDIUMGRAPH_HEIGHT, GRAPH_BPP);
		image_clear_alpha(&graph, (Color)color_bg, (Color)alpha_invisible);

		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH1_TOP, -20, 60, 0.5, "Minutes");
		graphs_create_rrd_snr(&graph, BIGGRAPH_GRAPH_LEFT, BIGGRAPH_GRAPH1_TOP, 0);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT -3, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 10, 10, SNR_SCALEFACTOR);
		graphs_create_valuescale(&graph, BIGGRAPH_GRAPH_LEFT + GRAPH_WIDTH + 16, BIGGRAPH_GRAPH1_TOP, GRAPH_HEIGHT, 10, 10, SNR_SCALEFACTOR);
		graphs_create_refscale(&graph, BIGGRAPH_GRAPH_LEFT, GRAPH_WIDTH, BIGGRAPH_GRAPH2_TOP, -20, 60, 0.5, "Minutes");
	} else {
		image_init(&graph, GRAPH_WIDTH, GRAPH_HEIGHT, GRAPH_BPP);
		graphs_create_rrd_errorsecs(&graph, 0, 0, 0);
	}

	image_save_png(&graph, imagefile);
	image_kill(&graph);
}
void graphs_create_rrd_snr(Image* graph, unsigned int x_start, unsigned int y_start, unsigned int paint_text) {
	unsigned int datapoint;
	unsigned int dataset;
	unsigned int graphpoint;
	unsigned int i;
	time_t starttime;
	time_t  endtime;
	unsigned long steptime;
	unsigned long dscount;
	rrd_value_t* rrddata;
	rrd_value_t* rrddatacopy;
	char **dsnames;
	float value;
	float value_min;
	float value_avg;
	float value_max;

	rrdstat_fetch_errorsecs(conf.rrdfile_snr, &rrddata, &starttime, &endtime, &steptime, &dscount, &dsnames);
	rrddatacopy=rrddata;

	unsigned int x_offset=0;
	unsigned int datapointscount=(endtime-starttime)/steptime;
	unsigned int stretchfactor=1;
	if (datapointscount<=GRAPH_WIDTH) {
		stretchfactor=(int)(GRAPH_WIDTH/datapointscount);
		x_offset=(GRAPH_WIDTH-(datapointscount*stretchfactor))/2;
	}
		
	image_paint_rect(graph, x_start + x_offset, y_start, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_bg);
	image_paint_grid2(graph, x_start + x_offset, y_start, GRAPH_SNR_GRID_WIDTH, GRAPH_SNR_GRID_HEIGHT, x_start + GRAPH_WIDTH, y_start + GRAPH_HEIGHT, (Color)color_graph_grid);

	for (datapoint=starttime+steptime; datapoint<endtime; datapoint+=steptime) {
		for (dataset=0; dataset<dscount; dataset++) {
			value=*(rrddatacopy++);
			graphpoint = (int)((datapoint -starttime)/steptime);
			if (dataset==0) {
				value_min=value;
			}
			if (dataset==1) {
				value_avg=value;
			}
			if (dataset==2) {
				value_max=value;
			}
		}
		for (i=0;i<stretchfactor;i++) {
			//snr_max
			image_paint_vert_line(graph, x_start + x_offset + (graphpoint*stretchfactor)+i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ (int)(SNR_FACTOR * value_max),0,GRAPH_HEIGHT) , y_start + GRAPH_HEIGHT -1, (Color)color_graph_snr_max);
			//snr_average
			image_paint_vert_line(graph, x_start + x_offset + (graphpoint*stretchfactor)+i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ (int)(SNR_FACTOR * value_avg),0,GRAPH_HEIGHT) , y_start + GRAPH_HEIGHT -1, (Color)color_graph_snr_avg);
			//snr_min
			image_paint_vert_line(graph, x_start + x_offset + (graphpoint*stretchfactor)+i - DROP_TONES, y_start + GRAPH_HEIGHT  - value_cap(1+ (int)(SNR_FACTOR * value_min),0,GRAPH_HEIGHT) , y_start + GRAPH_HEIGHT -1, (Color)color_graph_snr_min);
		}
	}

	for (dataset = 0; dataset < dscount; dataset++) {
		  free(dsnames[dataset]);
	}
	free(dsnames);
	free (rrddata);

	if (paint_text) {
		image_paint_text(graph, (x_start + 3) *64, (y_start + 3) *64, 0, 6*64, 7*64, conf.font, (Color)color_white, "errored seconds");
	}
}

#endif //rrd


void graphs_create_refscale(Image* graph, unsigned int x_start, unsigned int width, unsigned int y, int step, unsigned int pos_first_value, double scalefactor, char* unit) {
	int i;
	char* text;
	text=dmalloc(sizeof(char)*(10));
	if (step>0) {
		for (i=pos_first_value; i < width -1 ; i+=step) {
			sprintf(text, "%i", (int)(i*scalefactor));
			image_paint_text(graph, (x_start + i - DROP_TONES - (int)(strlen(text)*6/2)) *64, (y - 9) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
	} else if (step<0) {
		for (i=width-pos_first_value; i > 1 ; i+=step) {
			sprintf(text, "%i", (int)((width-i)*scalefactor));
			image_paint_text(graph, (x_start + i - DROP_TONES - (int)(strlen(text)*6/2)) *64, (y - 9) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
		}
	} else { //step==0
		message_write(MSGLEVEL_WARNING, "Won't make steps of size 0\n");
	}
	free(text);
	image_paint_text(graph, (x_start + width - (int)(strlen(unit)*5)) *64, (y - 9) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, unit);
}

void graphs_create_valuescale(Image* graph, unsigned int x, unsigned int y_start, unsigned int height, unsigned int step, unsigned int pos_first_value, double scalefactor) {
	unsigned int i;
	char* text;
	text=dmalloc(sizeof(char)*(10));
	for (i=pos_first_value; i < height -1 ; i+=step) {
		sprintf(text, "%i", (int)(i*scalefactor));
		image_paint_text(graph, (x - (int)(strlen(text)*5)) *64, (y_start - 4 + height - i) *64, 0, 6*64, 7*64, conf.font, (Color)color_black, text);
	}
	free(text);
}
#endif //graphic

