/**********************************************************************
 * $Id: graphic.c 177 2006-09-24 23:00:54Z timo $
 *
 * DMT-UX
 * DSL Modem Tool for Unix
 *
 * written by Timo Boettcher
 * inspired by A. Matthoefer
 * with help of Michael Beckers
 *
 * (C) 2006 Timo Boettcher
 * This is free software. You may redistribute copies of it under the
 * terms of the GNU General Public License (Version 2)
 * <http://www.gnu.org/licenses/gpl.html>.
 * There is NO WARRANTY, to the extent permitted by law.
 *
 *********************************************************************/

#include <string.h>
#include <math.h>
#include <png.h>
#include <ft2build.h>
#include FT_FREETYPE_H

#include "helper.h"
#include "colors.h"
#include "graphic.h"

void image_init(Image* image, unsigned int width, unsigned int height, unsigned int bpp) {
	if (bpp==4) {
		image->bpp = 4;
	} else {
		image->bpp = 3;
	}
	image->width = width;
	image->height = height;
	image->pixels = dmalloc(sizeof(unsigned char) * width * height * image->bpp);
}

void image_clear(Image* image, Color color_bg) {
	unsigned int x, y;
	for (y = 0; y < image->height; y++) {
		for (x = 0; x < image->width; x++) {
			image_paint_pixel(image, x, y, color_bg);
		}
	}
}

void image_clear_alpha(Image* image, Color color_bg, Color alpha) {
	unsigned int x, y;
	for (y = 0; y < image->height; y++) {
		for (x = 0; x < image->width; x++) {
			image_paint_pixel_alpha(image, x, y, color_bg, alpha);
		}
	}
}

void image_paint_rect(Image* image, unsigned int x_start, unsigned int y_start, unsigned int x_end, unsigned int y_end, Color color_bg) {
	unsigned int x, y;
	for (y = y_start; y < y_end; y++) {
		for (x = x_start; x < x_end; x++) {
			image_paint_pixel(image, x, y, color_bg);
		}
	}
}

void image_kill(Image* image) {
	free(image->pixels);
}





void image_paint_pixel(Image* image, unsigned int x, unsigned int y, Color color) {
	if (x >= 0 && y >= 0 && x < image->width && y < image->height) {
		memcpy(image->pixels + (y * image->width + x) * image->bpp, color, 3 * sizeof(unsigned char));
		if (image->bpp==4) {
			image->pixels[(y * image->width + x) * image->bpp + 3]= alpha_visible[0];
		}
	}
}

void image_paint_pixel_alpha(Image* image, unsigned int x, unsigned int y, Color color, Color alpha) {
	if (x >= 0 && y >= 0 && x < image->width && y < image->height) {
		memcpy(image->pixels + (y * image->width + x) * image->bpp, color, 3 * sizeof(unsigned char));
		if (image->bpp==4) {
			image->pixels[(y * image->width + x) * image->bpp+3]= alpha[0];
		}
	}
}

void image_paint_alpha(Image* image, unsigned int x, unsigned int y, Color alpha) {
	if (image->bpp==4) {
		if (x >= 0 && y >= 0 && x < image->width && y < image->height) {
			image->pixels[(y * image->width + x) * image->bpp+3]=alpha[0];
		}
	}
}

void image_paint_hor_line(Image* image, unsigned int x1, unsigned int x2, unsigned int y, Color color) {
	unsigned int i;
	for (i = x1; i <= x2; i++) {
		image_paint_pixel(image, i, y, color);
	}
}

void image_paint_vert_line(Image* image, unsigned int x, unsigned int y1, unsigned int y2, Color color) {
	unsigned int i;
	for (i = y1; i <= y2; i++) {
		image_paint_pixel(image, x, i, color);
	}
}

void image_paint_grid(Image* image, unsigned int xspacing, unsigned int yspacing, Color color) {
	unsigned int i;
	for (i = 0; i < image->height; i += yspacing) {
		image_paint_hor_line(image, 0, image->width - 1, i, color);
	}
	for (i = 0; i < image->width; i += xspacing) {
		image_paint_vert_line(image, i, 0, image->height - 1, color);
	}
}

void image_paint_grid2(Image* image, unsigned int x_start, unsigned int y_start, unsigned int x_spacing, unsigned int y_spacing, unsigned int x_end, unsigned int y_end, Color color) {
	unsigned int i;
	for (i = y_start; i < y_end; i += y_spacing) {
		image_paint_hor_line(image, x_start, x_end - 1, i, color);
	}
	for (i = x_start; i < x_end; i += x_spacing) {
		image_paint_vert_line(image, i, y_start, y_end - 1, color);
	}
}

void image_paint_xpm(Image* image, char** xpm, unsigned int x_start, unsigned int y_start) {
	unsigned int width, height;
	unsigned int number_colors;
	unsigned int charperpixel;
	XpmColors* xpm_colors;

	char* nexttoken;
	unsigned int tokencount;
	
	unsigned int i, x, y;
	char* xpmline;

	char** temp;
	char* red;
	char* blue;
	char* green;
	char* pixel;

	temp=dmalloc(sizeof(char*));
	//temp[0]=dmalloc(sizeof(char)*1);
	//temp[0][0]='\0';

	width=0;
	height=0;
	number_colors=0;
	charperpixel=0;
	tokencount=0;
	
	if (!xpm[0]) {
		message_write(MSGLEVEL_WARNING, "error parsing xpm: header not found\n");
	}
	xpmline=dmalloc((strlen(xpm[0])+1)*sizeof(char));
	strcpy(xpmline,xpm[0]);
	nexttoken=strtok(xpmline, " ");
	while (nexttoken) {
		tokencount++;
		switch(tokencount) {
		case 1:
			width=atoi(nexttoken);
			if (width==0) {
				message_write(MSGLEVEL_WARNING, "error parsing xpm: width empty\n");
				return;
			}
			break;
		case 2:
			height=atoi(nexttoken);
			if (height==0) {
				message_write(MSGLEVEL_WARNING, "error parsing xpm: height empty\n");
				return;
			}
			break;
		case 3:
			number_colors=atoi(nexttoken);
			if (number_colors==0) {
				message_write(MSGLEVEL_WARNING, "error parsing xpm: number_colors empty\n");
				return;
			}
			break;
		case 4:
			charperpixel=atoi(nexttoken);
			if (charperpixel==0) {
				message_write(MSGLEVEL_WARNING, "error parsing xpm: char_per_pixel empty\n");
				return;
			}
			break;
		default:
			message_write(MSGLEVEL_WARNING, "error parsing xpm: unknown header field\n");
		}
		nexttoken=strtok(NULL, " ");
	}
	free(xpmline);

	red=dmalloc(sizeof(char)*(3));
	green=dmalloc(sizeof(char)*(3));
	blue=dmalloc(sizeof(char)*(3));
	pixel=dmalloc(sizeof(char)*(charperpixel+1));
	xpm_colors=dmalloc(sizeof(XpmColors)*number_colors);
	for(i=0; i<number_colors; i++) {
		xpm_colors[i].character=dmalloc(sizeof(char)*(charperpixel+1));
		xpm_colors[i].color=dmalloc(sizeof(unsigned int)*(3));

		tokencount=0;
		xpmline=dmalloc((strlen(xpm[i+1])+1)*sizeof(char));
		strcpy(xpmline,xpm[i+1]);
		nexttoken=strtok(xpmline, "\t");
		while (nexttoken) {
			tokencount++;
			switch(tokencount) {
			case 1:
				//char that means a color
				memcpy(pixel,nexttoken,2*sizeof(char));
				strcpy(xpm_colors[i].character,nexttoken);
				break;
			case 3:
				//color that is meant by char
				if (strcmp(nexttoken,"None")!=0) {
					if (strlen(nexttoken)==7) {
						memcpy(red,nexttoken+1,2*sizeof(char));
						memcpy(green,nexttoken+3,2*sizeof(char));
						memcpy(blue,nexttoken+5,2*sizeof(char));
						red[2]='\0';
						green[2]='\0';
						blue[2]='\0';

						xpm_colors[i].color[0]=strtoul(red,temp,16);
						xpm_colors[i].color[1]=strtoul(green,temp,16);
						xpm_colors[i].color[2]=strtoul(blue,temp,16);
					} else {
						char* message = dmalloc(sizeof(char) * 1024);
						sprintf(message, "error parsing xpm: unknown color \"%s\"\n", nexttoken);
						message_write(MSGLEVEL_WARNING,message);
						free(message);
					}
				}

				break;
			}
			nexttoken=strtok(NULL, " ");
		}
		free(xpmline);
	}
	free(red);
	free(blue);
	free(green);
	free(pixel);
	//free(temp[0]);
	free(temp);
	pixel=dmalloc(sizeof(char)*3);
	for(y=0; y<height; y++) {
		xpmline=dmalloc((strlen(xpm[number_colors+1+y])+1)*sizeof(char));
		strcpy(xpmline,xpm[number_colors+1+y]);
		for(x=0; x<width; x++) {
			memcpy(pixel,xpmline + 2*x, 2 * sizeof(char));
			pixel[2]='\0';
			for(i=0; i<number_colors; i++) {
				if (strcmp(pixel,xpm_colors[i].character)==0) {
					memcpy(image->pixels + ((y_start + y) * image->width + x_start + x) * image->bpp, xpm_colors[i].color, 3 * sizeof(unsigned char));
					if (image->bpp==4) {
						if (i!=1) {
							memcpy(image->pixels + ((y_start + y) * image->width + x_start + x) * image->bpp + 3, alpha_visible, sizeof(unsigned char));
						}
					}
				}
			}
		}
		free(xpmline);
	}
	for(i=0; i<number_colors; i++) {
		free(xpm_colors[i].character);
		free(xpm_colors[i].color);
	}
	free(xpm_colors);
	free(pixel);
}

void image_paint_text(Image* image, unsigned int pos_x, unsigned int pos_y, int angle, unsigned int size_x, unsigned int size_y, char* font, Color color, char* text) {
	FT_Library library;
	FT_Face face;
	FT_GlyphSlot slot;
	FT_Matrix matrix;
	FT_Vector pen;
	FT_Error error;
	double angle2;

	unsigned int i, num_chars;

	num_chars = strlen(text);

	error = FT_Init_FreeType(&library);
	error = FT_New_Face(library, font, 0, &face);
	error = FT_Set_Char_Size(face, size_x, size_y, 100, 100);

	slot = face->glyph;
	

	angle2=( angle / 360 ) * 3.14159 * 2;
	angle2=0;
	matrix.xx = (FT_Fixed)( cos(angle2) * 0x10000L);
	matrix.xy = (FT_Fixed)(-sin(angle2) * 0x10000L);
	matrix.yx = (FT_Fixed)( sin(angle2) * 0x10000L);
	matrix.yy = (FT_Fixed)( cos(angle2) * 0x10000L);

	pen.x = pos_x;
	//pen position counts from bottom up, but we count top down
	pen.y = image->height*64 - pos_y - size_y;

	for (i = 0; i < num_chars; i++) {
		FT_Set_Transform(face, &matrix, &pen);

		error = FT_Load_Char(face, text[i], FT_LOAD_RENDER);

		image_paint_textbitmap(image, &slot->bitmap, slot->bitmap_left, image->height - slot->bitmap_top, color);

		pen.x += slot->advance.x;
		pen.y += slot->advance.y;
	}

	FT_Done_Face(face);
	FT_Done_FreeType(library);

}

void image_paint_textbitmap(Image* image, FT_Bitmap* bitmap, unsigned int x, unsigned int y, Color color) {
	FT_Int	i, j, p, q;
	FT_Int	x_max = x + bitmap->width;
	FT_Int	y_max = y + bitmap->rows;
	
	for (i = x, p = 0; i < x_max; i++, p++) {
		for (j = y, q = 0; j < y_max; j++, q++) {
			if (bitmap->buffer[q * bitmap->width + p]>127) {
				image_paint_pixel(image, i, j, color);
			}
		}
	}

}

void image_save_png(Image* image, char* filename) {
	unsigned int i;
	FILE* file = fopen(filename, "wb");

	if (!file) {
		char* message = dmalloc(sizeof(char) * 1024);
		sprintf(message, "could not write to file \"%s\"\n", filename);
		message_write(MSGLEVEL_WARNING,message);
		free(message);
		return;
	}

	png_structp png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, 0, 0, 0);
	png_infop png_info = png_create_info_struct(png_ptr);
	png_init_io(png_ptr, file);
	if (image->bpp==4) {
		png_set_IHDR(png_ptr, png_info, image->width, image->height, 8, PNG_COLOR_TYPE_RGB_ALPHA, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
	} else {
		png_set_IHDR(png_ptr, png_info, image->width, image->height, 8, PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_DEFAULT, PNG_FILTER_TYPE_DEFAULT);
	}
	unsigned char** rowarray = dmalloc(sizeof(unsigned char*) * image->height);
	for (i = 0; i < image->height; i++) {
		rowarray[i] = image->pixels + i * image->width * image->bpp;
	}
	png_set_rows(png_ptr, png_info, rowarray);
	png_write_png(png_ptr, png_info, PNG_TRANSFORM_IDENTITY, NULL);
	png_destroy_write_struct(&png_ptr, &png_info);
	free(rowarray);
	fclose(file);
}
